/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is ForumZilla 0.1.1.
 *
 * The Initial Developer of the Original Code is Zapogee, Inc.
 * Portions created by Zapogee, Inc. are Copyright (C) 2001
 * Zapogee, Inc.  All Rights Reserved.
 *
 * Contributor(s):
 *   Myk Melez, <myk@melez.com>, original author
 */

//****************************************************************************//
// constants and global variables

var gMailSession;

var NewFolderListener = {
    OnItemAdded: function(parentItem, item) {},
    OnItemRemoved: function(parentItem, item){},
    OnItemPropertyChanged: function(item, property, oldValue, newValue) {},
    OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {},
    OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {},
    OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue){},
    OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},

    OnItemEvent: function(folder, event) {
        var eventType = event.toString();
        if (eventType == "FolderCreateCompleted") {
            SetFolderPicker(folder.URI, GetFolderPicker().id);
            SetBusyCursor(window, false);
        }     
        else if (eventType == "FolderCreateFailed") {
            SetBusyCursor(window, false);
        }
    }
}


//****************************************************************************//
// event handlers

function load() {
    // Get preferences from the preferences service.
    var pollingInterval = nsPreferences.getIntPref("forumzilla.polling.interval", DEFAULT_POLLING_INTERVAL);
    var storageMode = nsPreferences.getIntPref("forumzilla.storage.mode", DEFAULT_STORAGE_MODE);
    var storageFolder = nsPreferences.copyUnicharPref("forumzilla.storage.folder", DEFAULT_STORAGE_FOLDER);

    // Apply preferences to the form fields.
    document.getElementById("polling.interval").value = pollingInterval;
    document.getElementById("storage.mode").selectedIndex = storageMode;
    SetFolderPicker(storageFolder, document.getElementById("msgNewFolderPicker").id);
    SetFolderPicker(storageFolder, document.getElementById("actionTargetFolder").id);

    // Make sure the right picker is showing.
    updatePickers(storageMode);
}

function unload() {
    // Get preferences from the form fields.
    var pollingInterval = document.getElementById("polling.interval").value;
    var storageMode = document.getElementById("storage.mode").value;
    var storageFolder;
    if (storageMode == FOLDER_STORAGE_MODE) {
        storageFolder = document.getElementById("actionTargetFolder").getAttribute('uri');
    }
    else if (storageMode == SUBFOLDERS_STORAGE_MODE) {
        storageFolder = document.getElementById("msgNewFolderPicker").getAttribute('uri');
    }

    // Save preferences back to the preferences service.
    nsPreferences.setIntPref("forumzilla.polling.interval", pollingInterval);
    nsPreferences.setIntPref("forumzilla.storage.mode", storageMode);
    nsPreferences.setUnicharPref("forumzilla.storage.folder", storageFolder);

    if (gMailSession)
        gMailSession.RemoveFolderListener(NewFolderListener);
}

function updatePickers(storageMode) {
    // When the storage mode changes, hides one picker, shows the other,
    // and sets the newly shown picker to the newly hidden one's folder value,
    // unless the folder is invalid for this storage mode, in which case
    // sets it to the default value.

    var folderPicker = document.getElementById("actionTargetFolder");
    var subfolderPicker = document.getElementById("msgNewFolderPicker");

    // Figure out which picker to hide and which to show.
    var hidePicker, showPicker;
    if (storageMode == FOLDER_STORAGE_MODE) {
        hidePicker = subfolderPicker;
        showPicker = folderPicker;
    }
    else if (storageMode == SUBFOLDERS_STORAGE_MODE) {
        hidePicker = folderPicker;
        showPicker = subfolderPicker;
    }

    // Hide and show the respective pickers.
    hidePicker.style.display = "none";
    showPicker.style.display = "block";

    // Update the newly shown picker with the appropriate value.
    var folderURI = validateFolder(hidePicker.getAttribute('uri'), storageMode)
                    || DEFAULT_STORAGE_FOLDER;
    SetFolderPicker(folderURI, showPicker.id);
}


//****************************************************************************//
// misc

function validateFolder(uri, storageMode) {
    // Makes sure the given storage folder URI is valid given the storage mode.
    // Folders are valid unless they're on an IMAP server that doesn't support
    // dual-use folders and we're in the wrong storage mode for the given folder
    // type (i.e. in folder mode with a folder that can only contain subfolders
    // or in subfolders mode with a folder that can only contain messages).

    var folder, server, imapServer, dualUseFolders;

    try {
        folder = GetMsgFolderFromUri(uri);
        server = folder.server;
        imapServer =
            server.QueryInterface(Components.interfaces.nsIImapIncomingServer);
        if (imapServer)
            dualUseFolders = imapServer.dualUseFolders;
    }
    catch(e) {}

    if (folder && server && imapServer && !dualUseFolders) {
        if (storageMode == SUBFOLDERS_STORAGE_MODE && !folder.canCreateSubfolders) {
            return null;
        }
        else if (storageMode == FOLDER_STORAGE_MODE && folder.canCreateSubfolders) {
            return null;
        }
    }
    return uri;
}

/*
 XXX Here's a way of getting the default account's root folder:
     Perhaps we should do this instead of having the DEFAULT_STORAGE_FOLDER
     constant, as this is more likely to be correct more of the time.
var defaultFolder = GetDefaultAccountRootFolder();
var checkCreateSubfolders = null;
if (defaultFolder)
    checkCreateSubfolders = defaultFolder.canCreateSubfolders;

if (checkCreateSubfolders)
    destinationFolder = defaultFolder;
*/


function NewFolderOkCallback(name,uri) {
    var msgFolder = GetMsgFolderFromUri(uri, true);

    var imapFolder = null;
    try {
        imapFolder = msgFolder.QueryInterface(Components.interfaces.nsIMsgImapMailFolder);
    }
    catch(e) {}

    var mailSessionContractID = "@mozilla.org/messenger/services/session;1";
    if (imapFolder) { // IMAP folder creation is asynchronous.
        if (!gMailSession)
            gMailSession =
              Components
                .classes["@mozilla.org/messenger/services/session;1"]
                  .getService(Components.interfaces.nsIMsgMailSession);
        try {
            var nsIFolderListener = Components.interfaces.nsIFolderListener;
            var notifyFlags = nsIFolderListener.event;
            gMailSession.AddFolderListener(NewFolderListener, notifyFlags);
        } 
        catch (ex) {
            dump("Error adding to session: " +ex + "\n");
        }
    }

    if (imapFolder)
        SetBusyCursor(window, true);
  
    msgFolder.createSubfolder(name, GetMsgWindow());
    
    if (!imapFolder) {
        // XXX This seems prone to failure, since a folder's URI is not always
        // exactly the same as their parent folder's URI + / + their encoded name.
        var folder = uri + "/" + encodeURIComponent(name);
        SetFolderPicker(folder, GetFolderPicker().id);
    }
}

function GetFolderPicker() {
    var storageMode = document.getElementById("storage.mode").value;
    var picker;
    if (storageMode == FOLDER_STORAGE_MODE)
        picker = document.getElementById("actionTargetFolder");
    else if (storageMode == SUBFOLDERS_STORAGE_MODE)
        picker = document.getElementById("msgNewFolderPicker");
    return picker;
}

function GetSelectedMsgFolders() {
    var picker = GetFolderPicker();
    var uri = picker.getAttribute('uri');
    var folder = GetMsgFolderFromUri(uri);
    return [folder];
}
